/*
 * Copyright (c) 2000 - 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _SEC_CRYPTO_SVC_H
#define _SEC_CRYPTO_SVC_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdbool.h>

/*typedef enum
{
    SEC_CRYPTO_CIPHER_NULL = 0,
    SEC_CRYPTO_CIPHER_AES_CBC_128,
    SEC_CRYPTO_CIPHER_SEED_CBC,
    SEC_CRYPTO_CIPHER_AES_CBC_192,
    SEC_CRYPTO_CIPHER_AES_CBC_256,
    SEC_CRYPTO_CIPHER_AES_CTR_128,
    SEC_CRYPTO_CIPHER_AES_ECB_128,
    SEC_CRYPTO_CIPHER_AES_ECB_192,
    SEC_CRYPTO_CIPHER_AES_ECB_256
} SecCryptoCipherAlg;

typedef enum
{
    SEC_SUCCESS = 0,
    SEC_CRYPTO_ERROR_1, //algorithm error
    SEC_CRYPTO_ERROR_2, //init error
    SEC_CRYPTO_ERROR_3, //update error
    SEC_CRYPTO_ERROR_4, //final error
    SEC_CRYPTO_ERROR_5, //wrong param
    SEC_CRYPTO_ERROR_6, //Memory alloc
    SEC_CRYPTO_ERROR_7, //Internal error
} SecError;
*/

#ifndef IN
#define IN
#endif
#ifndef OUT
#define OUT
#endif

#ifndef NULL
#define NULL 0
#endif
#ifndef BOOL
#define BOOL bool
#endif
#ifndef ULONG
#define ULONG unsigned int
#endif
#ifndef UINT8
#define UINT8 unsigned char
#endif
#ifndef UINT32
#define UINT32 unsigned int
#endif

//#define EVP_ERROR         0
//#define SEC_CRYPTO_ENCRYPT        1
//#define SEC_CRYPTO_DECRYPT        0
#define SEC_DUK_SIZE            16
//#define SEC_CRYPTO_KEY_LENGTH     16
//#define SEC_CRYPTP_ARR_LENGTH     1024
#define SEC_FRAME_OSP_KEY       "uniqueKey"
//#define SHA1_DIGEST_VALUE_LEN     20
#define SHA256_DIGEST_VALUE_LEN     32
//#define KDF_KEYLEN            16
#define HASH_LEN SHA256_DIGEST_VALUE_LEN
//#define SEC_KEYMGR_FEK_SIZE       16

#define CS_ERROR_NONE           0
#define CS_ERROR_BAD_ALLOC     -1
#define CS_ERROR_INTERNAL      -2
#define CS_ERROR_INVALID_PARAM -3

/*------ Base64 Encoding Table ------*/
static const char Base64EncodingTable[] = {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
	'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
	'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
	'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
	'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
	'w', 'x', 'y', 'z', '0', '1', '2', '3',
	'4', '5', '6', '7', '8', '9', '+', '/'
};

/**
    * This function provides Device UniqueKey for crypto with Hash
    * @return       This function returns TRUE on success
    * @param[in]        uLen    Length of Device UniqueKey
    * @param[in,out]    pCek    Device UniqueKey(Hashed)
*/
bool SecFrameGeneratePlatformUniqueKey(IN UINT32 uLen, IN OUT UINT8 *pCek);

char *Base64Encoding(const char *data, int size);

char *GetDuid(int size);

/*
 *  Password based derivation routines with platform key
 *
 *  @remarks Returned @a key should be freed by caller after use.
 *
 *  @param[in]  pass     password used in the key derivation. Shouldn't be NULL
 *  @param[in]  passlen  length of @a password to use
 *  @param[in]  keylen   length of key to make which should be bigger than 0
 *  @param[out] key      derived key with @a pass and platform key. Shouldn't be NULL
 *
 *  @return CS_ERROR_NONE on success, otherwise negative error codes
 *  #retval CS_ERROR_NONE           Success
 *  #retval CS_ERROR_BAD_ALLOC      Memory allocation failed to new key
 *  #retval CS_ERROR_INTERNAL       Internal error which should not be happened
 *  #retval CS_ERROR_INVALID_PARAM  Invalid parameter. @a pass and @a key shouldn't be NULL
 *                                  and keylen should be bigger than 0
 */
int cs_derive_key_with_pass(const char *pass, int passlen, int keylen,
							unsigned char **key);

#ifdef __cplusplus
}
#endif

#endif  // _SEC_CRYPTO_SVC_H

